@@ -9,18 +9,18 @@ cd $_DIR
 git pull
 
 ./clippy.sh
-./example.sh > example.out
 
 npx @rmw/md-include .markdown.json
 
 cargo set-version --bump patch
 
-version=$(cat Cargo.toml|grep "^version"|awk -F\" '{print $2}')
+version=v$(cat Cargo.toml|grep "^version"|awk -F\" '{print $2}')
 
 git add -u
-git commit -m "v$version"
-git tag v$version
-git push
+git commit -m $version
+git tag $version
+git push origin $version 
+git push 
 
 cargo +nightly publish
 
