@@ -3,8 +3,11 @@ pub use crate::{
   dollar,
   env::Env,
   env_rw, flag,
-  r#type::{Bin, Str},
-  rw, Db,
+  r#type::{Bin, FromMdbx, Str, ToAsRef},
+  rw,
+  tx::PtrTx,
+  val::MDBX_val,
+  val_bytes, Db,
 };
 pub use ffi::mdbx_version;
 pub use mdbx_proc::mdbx;
